/**
 *  @file rfcomm_scan.c
 *
 *  RFCOMM scan - a Bluetooth(tm) RFCOMM channel scanner
 *
 *  @author Collin R. Mulliner <collin@betaversion.net>
 *
 *  (c) Collin R. Mulliner
 *
 *  web: www.betaversion.net/btdsd/
 */

/*
 * This file is part of rfcomm_scan
 *
 * rfcomm_scan is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * rfcomm_scan is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with rfcomm_scan; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/poll.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>

#include <asm/types.h>
#include <asm/byteorder.h>

#include <bluetooth/bluetooth.h>
#include <bluetooth/l2cap.h>
#include <bluetooth/rfcomm.h>
#include <bluetooth/hci.h>
#include <bluetooth/hci_lib.h>

/**
 *  @brief scan RFCOMM Channels
 *
 *  @param dstAddr bluetooth address (format XX:XX:XX:XX:XX:XX)
 *  @param s       start RFCOMM Channel
 *  @param e       end RFCOMM Channel
 *  @param o       only show open channels
 *
 *  @returns 0 on success -1 on failure
 */
int rfcomm_scan(char *dstAddr, char *srcAddr,  unsigned short int s, unsigned short int e, int o)
{
	struct sockaddr_rc sa;
	int sock;
	bdaddr_t src, dst;
	unsigned int i;

	
	baswap(&dst, strtoba(dstAddr));
	baswap(&src, strtoba(srcAddr));
	
	for (i = s; i <= e; i++) {
	
		sock = socket(PF_BLUETOOTH, SOCK_STREAM, BTPROTO_RFCOMM);

		if (sock >= 0) {
			sa.rc_family = AF_BLUETOOTH;
			sa.rc_channel = 0;
		
			if (bacmp(&src, BDADDR_ANY) != 0) {
				sa.rc_bdaddr = src;
				if (bind(sock, (struct sockaddr *) &sa, sizeof(sa)) < 0) {
					perror("bind() failed");
					return(-1);
				}
			}

			sa.rc_channel = i;
			sa.rc_bdaddr = dst;

			if (connect(sock, (struct sockaddr *) &sa, sizeof(sa)) != 0) {
				if (o == 0) {
					printf("rfcomm: %0.2d closed\n", i);
				}
			}
			else {
				printf("rfcomm: %0.2d open\n", i);
			}
		}
		else {
			perror("socket() failed\n");
		}

		close(sock);
	}
	
	return(0);
}

void usage()
{
	printf("\nrfcomm_scan v0.1 (c) Collin R. Mulliner <collin@betaversion.net>\nusage:\n\trfcomm_scan [-S <bd_addr>] [-o] "\
			"[-s <channel>] [-e <channel>] <bd_addr>\n\noptions:\n"\
			"\tS\t: source BDADDR (local)\n"\
			"\to\t: only show open channels\n"\
			"\ts\t: start channel (1-30)\n\te\t: end channel (1-30)\n\n"\
			"Note:\n"\
			"\tMake sure the host running rfcomm_scan is configured as YOU want, if authentication\n"\
			"\tand/or encryption is on/off active pairings will be used or new pairings will start!\n");
}

int main(int argc, char **argv)
{
	char srcAddr[20];
	int opt;
	int startc = 1;
	int endc = 30;
	int o = 0;
		

	memcpy(srcAddr, "00:00:00:00:00:00\0", 18);
	while ((opt = getopt(argc, argv,"S:e:s:o")) != EOF) {
	
		switch(opt) {
		case 'S':
			strncpy(srcAddr, optarg, 20);
			break;
		case 'o':
			o = 1;
			break;
		case 's':
			startc = atoi(optarg);
			if (startc < 1 || startc > 30) {
				usage();
				exit(-1);
			}
			break;
		case 'e':
			endc = atoi(optarg);
			if (endc < 1 || endc > 30) {
				usage();
				exit(-1);
			}
			break;
		default:
			usage();
			exit(-1);
		}
	}

	if (!(argc - optind)) {
		usage();
		exit(-1);
	}

	if (startc > endc) {
		usage();
		exit(-1);
	}
	
	exit(rfcomm_scan(argv[optind], srcAddr, startc, endc, o));
}
